# -*- python -*-
# $RCSfile: reporter_GUI.py,v $
# $Revision: 1.41 $
# $Author: reida $
# $Date: 2004/10/07 18:42:07 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 



# GUI part of error reporting machine.

from SWIG.common import switchboard
from common import debug
from common import excepthook
from common import guitop
from common import mainthread
from common import thread_enable
from common.IO import mainmenu
from common.IO import oofmenu
from common.IO import reporter
from common.IO import reportermenu
from common.IO.GUI import fileselector
from common.IO.GUI import gfxmenu
from common.IO.GUI import oof_mainiteration
from common.IO.GUI import subWindow
from gtk import *
import Queue
import string
import sys
import traceback
import os
import time

OOF = mainmenu.OOF

# Flag for indicating if we should auto_raise. 
_message_window_auto_raise = 1

# Counter, counts how many message windows are up.  Used to determine
# if auto-raising is required.
_message_windows_up = 0

class MessageWindow(subWindow.SubWindow):
    count = 1
    def __init__(self):
        debug.mainthreadTest()
        self.menu_name = "MessageWindow_%d" % MessageWindow.count
        self.title = "OOF2 Messages %d" % MessageWindow.count
        self.windows_menu_name = "Message_%d" % MessageWindow.count
        
        subWindow.SubWindow.__init__(
            self, title=self.title, menu=self.menu_name)

        # We are locally responsible for the windows submenu items.
        self.gtk.connect("destroy", self.destroy)

        # raise_window function is provided by the SubWindow class.
        OOF.Windows.Messages.addItem(
            oofmenu.OOFMenuItem(
            self.windows_menu_name,
            help="Raise Message window %d." % MessageWindow.count,
            cli_only=0, no_log=1,
            gui_callback=self.raise_window) )
            
        MessageWindow.count += 1

        global _message_windows_up
        _message_windows_up += 1

        # Control box, with buttons.  These could be menu items.
        controlbox = GtkFrame()
        controlbox.set_shadow_type(SHADOW_OUT)
        self.mainbox.add(controlbox, expand=0, fill=0)
        controlinnards = GtkVBox()
        controlbox.add(controlinnards)
        buttonbox = GtkHBox()
        controlinnards.add(buttonbox, expand=0, padding=2)
        savebutton = GtkButton("Save...")
        guitop.top().tooltip(savebutton, "Save the contents of this window to a file.")
        buttonbox.add(savebutton, expand=0, fill=0, padding=2)
        savebutton.connect('clicked', self.saveButtonCB)
        buttonbox.add(GtkHBox(), expand=1, fill=1) # filler
        self.button_dict = {}
        self.signal_dict = {}
        for m in reporter.messageclasses:
            button = GtkCheckButton(m)
            buttonbox.add(button, fill=0, expand=0, padding=2)
            self.signal_dict[m] = button.connect("clicked", self.button_click)
            self.button_dict[m] = button
            guitop.top().tooltip(button, "Show or hide "+
                                 reporter.messagedescriptions[m])

        messagepane = GtkScrolledWindow()
        messagepane.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
        self.mainbox.add(messagepane)

        self.messages = GtkText()
        self.messages.set_editable(0)
        self.messages.set_word_wrap(1)
        self.messages.set_usize(600,150)
        messagepane.add(self.messages)

        self.state_dict = {}
        for m in reporter.messageclasses:
            self.state_dict[m]=1
            
        self.sbcb = switchboard.requestCallback(
            "write message", self.write_message)

        self.draw()

    def destroy(self, gtkobj):
        global _message_window_auto_raise
        global _message_windows_up
        _message_window_auto_raise = None
        _message_windows_up -= 1
        switchboard.removeCallback(self.sbcb)
        OOF.Windows.Messages.removeItem(self.windows_menu_name)
        

    def draw(self):
        self.gtk.show_all()
        for (m,v) in self.state_dict.items():
            b = self.button_dict[m]
            b.signal_handler_block(self.signal_dict[m])
            if v:
                b.set_active(1)
            else:
                b.set_active(0)
            b.signal_handler_unblock(self.signal_dict[m])
        self.refresh()

    def refresh(self):
        debug.mainthreadTest()
        self.messages.freeze()
        self.messages.backward_delete(self.messages.get_length())
        for m in reporter.messagemanager.all_messages():
            if self.state_dict[m[1]]:
                self.messages.insert(None, None, None, m[0]+"\n")
        # This pair of thaw/freeze calls is apparently required,
        # otherwise the message window goes into an infinite loop.
        self.messages.thaw()
        self.messages.freeze()
        self.messages.set_position(self.messages.get_length())
        self.messages.thaw()

    # After each button click, set your local state.
    def button_click(self, gtkobj):
        for (m,b) in self.button_dict.items():
            if b.active:
                self.state_dict[m]=1
            else:
                self.state_dict[m]=0
        self.refresh()

    def write_message(self, message_tuple):
        debug.mainthreadTest()
        # Can use thaw/freeze here if its too flickery.
        if self.state_dict[message_tuple[1]]:
            self.messages.insert(None, None, None, message_tuple[0]+"\n")

    def saveButtonCB(self, button):
        file = fileselector.fileSelector(ident='SaveMessages',
                                         title='Save Messages')
        if file is not None:
            mode = fileselector.get_file_mode(file)
            if mode is not None:
                # The arguments for the Save command are constructed
                # from the same list of names as the state_dict, so we
                # just have to pass the dict here.
                OOF.File.Save.Messages.callWithDefaults(filename=file,
                                                        mode=mode,
                                                        **self.state_dict)
                                          
            
# Turn off the command-line outputs of the message manager.
for m in reporter.messageclasses:
    reporter.messagemanager.set_flag(m, None)

defaultfont = load_font("-*-fixed-*-*-*-*-*-120-*-*-*-*-iso8859-*")


class WarningPopUp:
    # Argument "message" is a single string for the second GtkLabel.
    def __init__(self, message):
        debug.mainthreadTest()
        self.gtk = GtkDialog()
        self.gtk.set_title("OOF2 Warning")
        self.gtk.set_usize(250,-1)

        self.gtk.vbox.set_spacing(3)

        self.gtk.vbox.add(GtkLabel("WARNING"))
        self.gtk.vbox.add(GtkLabel(message))

        okbutton = GtkButton("OK")
        self.gtk.action_area.add(okbutton)
        okbutton.connect("clicked", self.ok)

        nomorebutton = GtkButton("Don't warn again")
        self.gtk.action_area.add(nomorebutton)
        nomorebutton.connect("clicked", self.nomore)

        self.gtk.show_all()

    def ok(self, gtk):
        debug.mainthreadTest()
        self.gtk.destroy()

    def nomore(self, gtk):
        OOF.Help.Popup_warnings(0)
        self.gtk.destroy()
        
def _warning_pop_up(message):
    if reporter.messagemanager.get_warning_pop_up():
        WarningPopUp(message)

_warning_sb_callback = switchboard.requestCallback(
    "messagemanager warning", _warning_pop_up)


# The error pop up optionally allows you to view the traceback,
# and save it to a file.  The file is automatically named.
class ErrorPopUp:
    def __init__(self, type, value, tbacklist):
        global defaultfont
        debug.mainthreadTest()
        self.font = defaultfont
        self.tb_strings = []

        self.answer = None
        
        self.datestampstring = time.strftime("%Y %b %d %H:%M:%S %Z")
        self.gtk = GtkDialog()
        self.gtk.set_title("OOF2 Error")
        self.gtk.set_usize(500, -1)

        self.gtk.connect("destroy", self.destroy)
        
        self.gtk.vbox.set_spacing(3)
        classname = string.split(str(type),'.')[-1]
        self.error_string = traceback.format_exception_only(type, value)
        self.gtk.vbox.add(GtkLabel("ERROR"), expand=0, fill=0)

        # The error message is displayed in a GtkText object instead
        # of a GtkLabel because the formatting requires a monospaced
        # fonts.  At least until we learn how to manipulate fonts,
        # we're stuck with proportional fonts in labels and monospaced
        # fonts in texts.  Anyhow, putting the message in a text
        # object allows it to be selected and copied, which could be
        # handy.
        outererrbox = GtkHBox()
        self.gtk.vbox.add(outererrbox, expand=0, fill=1)
        errscroll = GtkScrolledWindow()
        errscroll.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
        outererrbox.add(errscroll, padding=8)
        self.errbox = GtkText()
        errscroll.add_with_viewport(self.errbox)
        self.errbox.set_editable(0)
        self.errbox.set_word_wrap(TRUE)  # wrapping messes up the formatting
        for e in self.error_string:
            self.errout(e)
        
        self.okbutton = GtkButton("OK")
        self.okbutton.connect("clicked", self.ok)
        self.gtk.action_area.add(self.okbutton)

        self.abortbutton = GtkButton("Abort")
        self.abortbutton.connect("clicked", self.abort)
        self.gtk.action_area.add(self.abortbutton)

        self.tracebutton = GtkButton("View Traceback")
        self.tracebutton.connect("clicked", self.trace)
        self.gtk.action_area.add(self.tracebutton)

        self.savebutton = GtkButton("Save Traceback")
        self.savebutton.connect("clicked", self.savetrace)
        self.gtk.action_area.add(self.savebutton)

        self.scroll = GtkScrolledWindow()
        self.scroll.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
        
        self.tracepane = GtkText()
        self.tracepane.set_editable(0)
        self.tracepane.set_word_wrap(1)

        self.frame = GtkFrame()
        self.gtk.vbox.add(self.frame)

        # Scroll not added to the frame until traceback button is pressed.
        self.scroll.add(self.tracepane)

        self.gtk.set_modal(1)

        tblist = traceback.format_list(tbacklist)
        if tblist:
            for s in tblist:
                self.tb_strings.append(s)
                self.textout(s)
        else:
            self.savebutton.set_sensitive(0)
            self.tracebutton.set_sensitive(0)

        self.gtk.show_all()

    def ok(self, gtk):
        debug.mainthreadTest()
        self.answer = 1
        self.gtk.destroy()

    def abort(self, gtk):
        debug.mainthreadTest()
        self.answer = 0
        self.gtk.destroy()

    def destroy(self, gtk):
        debug.mainthreadTest()
        if self.answer is None:
            self.answer = 0
            
    def trace(self, gtk):
        debug.mainthreadTest()
        c = self.frame.children()
        if len(c)==0:
            self.frame.add(self.scroll)
        else:
            self.frame.remove(c[0])
        self.gtk.show_all()

    def savetrace(self, gtk):
        fname = "traceback_oof."+str(os.getpid())
        try:
            fobj = file(fname, "a+")
            fobj.write("\n%s\n" % self.datestampstring)
            fobj.writelines(self.tb_strings)
            fobj.writelines(self.error_string)
            #
            self.errout("\nTraceback written to %s.\n" % fname)
        except IOError:
            self.errout("\nUnable to write traceback.\n")
            
    def textout(self, string):
        debug.mainthreadTest()
        self.tracepane.insert(self.font, None, None, string)

    def errout(self, string):
        debug.mainthreadTest()
        self.errbox.insert(self.font, None, None, string)

# Blocking function, which raises an error pop up, blocks, and
# returns "1" if the "OK" button was pressed, and "0" otherwise.
def errorpopup_(e_type, e_value, tbacklist):
    e = ErrorPopUp(e_type, e_value, tbacklist)
    while e.answer is None:
        oof_mainiteration.mainiteration()
    return e.answer
    

# This gets reassigned at import time, but other imports can cause
# errors, so it needs to check if the mainloop is running.
def _gui_printTraceBack(e_type, e_value, tbacklist):
    if guitop.mainLoop():
        # Transfer control to the main thread to report errors in the GUI.
        # debug.fmsg("Calling errorpopup on the main thread.")
        res = mainthread.runBlock(errorpopup_, (e_type, e_value, tbacklist))
        if res==0:
            from common.IO.GUI import quit
            if mainthread.runBlock(quit.quitQuery):
                os.abort()
    else:
        excepthook._printTraceBack(e_type, e_value, tbacklist)

excepthook.printTraceBack = _gui_printTraceBack

##excepthook.assign_excepthook(_gui_excepthook)

def newMessageWindow():
    return MessageWindow()

# Override the nonGUI function called by the message-window-creating
# menu callback.
reporter._new_messages = newMessageWindow

# Module-level switchboard callback for auto-raising.  Takes an
# argument because the signal we're using sends the message.
def _auto_raise(message_tuple):
    global _message_window_auto_raise
    global _message_windows_up
    if _message_window_auto_raise and (_message_windows_up==0):
        OOF.Windows.Messages.New()

switchboard.requestCallback("write message", _auto_raise)

############################

def _saveMessages_GUI(menuitem):
    file = fileselector.fileSelector(
        ident='SaveMessages',
        title='Save Messages',
        params=[menuitem.get_arg(name) for name in reporter.messageclasses])
    if file is not None:
        mode = fileselector.get_file_mode(file)
        if mode is not None:
            menuitem.callWithDefaults(filename=file, mode=mode)

OOF.File.Save.Messages.add_gui_callback(_saveMessages_GUI)
