# -*- python -*-
# $RCSfile: workerGUI.py,v $
# $Revision: 1.13 $
# $Author: reida $
# $Date: 2004/07/23 14:48:59 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov.

from common import debug
from common import parallel_enable
from common import mainthread
from common import thread_enable
from common import worker



# GUIThreadedWorkerBlock is called when running a script in GUI mode.
# The script is running on a subthread already, and commands in the
# script need to be executed sequentially, so we have to wait for each
# one to finish.

class GUIThreadedWorkerBlock(worker.ThreadedWorker):
    def meanwhile(self):
        ## debug.fmsg()
        self.worker.join()              # waits for child thread to finish


## parallel worker classes start here

class GUIParallelThreadedWorker(worker.ThreadedWorker, worker.ParallelWork):
    def __init__ (self, menuitem, kwargs):
        self.worker = worker.ParallelThreadedWorkerCore(menuitem, kwargs)
        threadmanager.threadManager.newWorker(self.worker)
        worker.ParallelWork.__init__(self, menuitem, kwargs)
        


class GUIParallelThreadedWorkerBlock(GUIThreadedWorkerBlock,
                                     worker.ParallelWork):
    def __init__ (self, menuitem, kwargs):
        self.worker = worker.ParallelThreadedWorkerCore(menuitem, kwargs)
        threadmanager.threadManager.newWorker(self.worker)
        worker.ParallelWork.__init__(self, menuitem, kwargs)
        
## parallel worker classes end here


def getThreadedWorker(menuitem, argdict):
    if not thread_enable.query():
        ## serial workers
        return worker.NonThreadedWorker(menuitem, argdict)
    if mainthread.mainthread():
        return worker.ThreadedWorker(menuitem, argdict)
    return GUIThreadedWorkerBlock(menuitem, argdict)

worker.getThreadedWorker = getThreadedWorker

def getThreadedParallelWorker(menuitem, argdict):
    ## parallel workers
    ## only called by front-end
    ## back-end is running on text-mode, without any text
    ## progress bars
    if not thread_enable.query():
        ## if GUI is not in place yet, do not 
        ## launch a thread. Instead run a serial
        ## parallel worker
        return worker.ParallelWorker(menuitem, argdict)
    if mainthread.mainthread():
        ## if main thread launches a parallel thread
        ## launch a GUI worker
        return GUIParallelThreadedWorker(menuitem, argdict)
    ## if subthread is attempting to launch a thread,
    ## block the subthread, until, sub-subthread joins
    return GUIParallelThreadedWorkerBlock(menuitem, argdict)


worker.getThreadedParallelWorker = getThreadedParallelWorker
