# -*- python -*-
# $RCSfile: mainmenu.py,v $
# $Revision: 1.116 $
# $Author: langer $
# $Date: 2005/02/08 21:57:19 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov.

#
#  The main oof menu -- with recording capability and so forth.

from SWIG.common import ooferror
from SWIG.common import switchboard
from SWIG.common.IO import threadstate
from common import debug
from common import thread_enable
from common import utils
from common.IO import oofmenu
from common.IO import parameter
from common.IO import reporter
from common.IO import scriptloader
from common.IO.gfxmanager import gfxManager
from common.IO import xmlmenudump
from types import StringType, FloatType, IntType
import code
import sys

# Parameter = parameter.Parameter
StringParameter = parameter.StringParameter
IntParameter = parameter.IntParameter
FloatParameter = parameter.FloatParameter

OOFMenuItem = oofmenu.OOFMenuItem
OOFRootMenu = oofmenu.OOFRootMenu
CheckOOFMenuItem = oofmenu.CheckOOFMenuItem

OOF = OOFRootMenu(
    'OOF',
    discussion=xmlmenudump.loadFile('DISCUSSIONS/common/IO/oof.xml'))

## File menu

_filemenu = OOF.addItem(OOFMenuItem(
    'File',
    help="Commands for saving and loading data, and quitting."))

_loadmenu = _filemenu.addItem(OOFMenuItem(
    'Load',
    help="Commands for loading datafiles and scripts.",
    discussion="<para>Commands to load datafiles and scripts.</para>"))

def OLDloadscript(menuitem, filename):
    if filename is not None:
##        menuitem.root().haltLog()       # don't re-log commands in the file
        debug.fmsg('reading', filename, 'in thread',
                   threadstate.findThreadNumber())
        utils.OOFexecfile(filename)
        debug.fmsg('finished reading', filename)
##        menuitem.root().resumeLog()

class PScriptLoader(scriptloader.ScriptLoader):
    # A ScriptLoader that supports a progress bar.
    def __init__(self, filename, progbar):
        self.progressbar = progbar
        scriptloader.ScriptLoader.__init__(self, filename,
                                         sys.modules['__main__'].__dict__)  
    def progress(self, current, total):
        self.progressbar.set_progress((1.0*current)/total)
        self.progressbar.set_message("%d/%d lines" % (current, total))
    def stop(self):                     # called by ScriptLoader to abort loop
        return self.progressbar.query_stop()
    def done(self):                     # called by ScriptLoader when finished
        if self.error:
            self.progressbar.set_failure()
        else:
            self.progressbar.set_success()
        scriptloader.ScriptLoader.done(self)

def loadscript(menuitem, filename):
    if filename is not None:
        debug.fmsg('reading', filename, 'in thread',
                   threadstate.findThreadNumber())
        interp = PScriptLoader(filename, menuitem.getProgressBar("continuous"))
        interp.run()
        debug.fmsg('finished reading', filename)

_loadmenu.addItem(OOFMenuItem(
    'Script',
    callback=loadscript,
    params=[StringParameter('filename', 'logfile', tip="Name of the file.")],
    no_log=1,
    ellipsis=1,
    threadable=oofmenu.THREADABLE_GUI,
    accel='l',
    help="Execute a Python script.",
    discussion=xmlmenudump.loadFile('DISCUSSIONS/common/IO/menu-loadscript.xml')
    ))

# "Data" files are distinguished from scripts in that they're not read
# by the Python interpreter directly, and therefore can't contain
# arbitrary Python code.  They can only contain menu commands from the
# OOF.LoadData menu.  The command arguments can be constants or
# variables and functions that are defined in the main OOF namespace,
# or lists and tuples thereof.

def loaddata(menuitem, filename):
    if filename is not None:
        from common.IO import datafile
        debug.fmsg('loading', filename)
        datafile.readDataFile(filename, OOF.LoadData,
                              menuitem.getProgressBar("continuous"))
        debug.fmsg('done loading', filename)

_loadmenu.addItem(OOFMenuItem(
    'Data',
    callback=loaddata,
    threadable=oofmenu.THREADABLE,
    params=[StringParameter('filename', tip="Name of the file.")],
    ellipsis=1,
    help="Load a data file.",
    discussion=xmlmenudump.loadFile('DISCUSSIONS/common/IO/menu-loaddatafile.xml')
    ))

OOF.addItem(oofmenu.OOFMenuItem(
    "LoadData", secret=1,
    help="Commands used in data files.",
    discussion=xmlmenudump.loadFile('DISCUSSIONS/common/IO/menu-loaddata.xml')
    ))


def saveLog(menuitem, filename):
    file = open(filename, 'w')
    menuitem.root().saveLog(file)
    file.close()

_savemenu = _filemenu.addItem(
    OOFMenuItem('Save', help='Create data files and scripts.'))

_savemenu.addItem(OOFMenuItem(
    'Python_Log',
    callback=saveLog,
    params=[StringParameter('filename', tip="Name of the file.")],
    accel='s',
    ellipsis=1,
    no_log=1,
    help="Save the current session as a Python script.",
    discussion=xmlmenudump.loadFile('DISCUSSIONS/common/IO/menu-savelog.xml')
    ))

#################################

def quitCmd(menuitem):
    from common import quit
    quit.quit()

_filemenu.addItem(OOFMenuItem(
    'Quit',
    callback=quitCmd,
    accel='q',
    help="Don't give up so easily!",
    discussion=xmlmenudump.loadFile('DISCUSSIONS/common/IO/menu-quit.xml'),
    threadable = oofmenu.UNTHREADABLE,
    no_log=1,
    )) 


##def dwiw(*args):
##    print "I'm sorry, Dave, I can't do that."
##_filemenu.addItem(OOFMenuItem('Do_What_I_Want', callback=dwiw))


##################################

## Subwindows menu.

_windowmenu = OOFMenuItem(
    'Windows',
    help="Menus for opening and raising windows.")
OOF.addItem(_windowmenu)

def openGfx(menuitem):
    window = gfxManager.openWindow()
    window.draw()
    window.zoomFillWindow()

    
def dummy(menuitem): pass   # Dummy callback, so trivial menu items get logged.

# Add an entry for the main window, which, when clicked, raises it.
_windowmenu.addItem(OOFMenuItem("OOF2", callback=dummy,
                                help="Raise the main OOF2 window.",
                                discussion="<para>Any sub-window of &oof2; has this menu available. So, it's a good way to locate the main &oof2; window, if it's out of sight.</para>"))

_graphicsmenu = OOFMenuItem(
    'Graphics',
    help="Graphical display of &oof2; objects.",
    discussion="""<para>
    Graphics windows are discussed in <xref
    linkend='Chapter:Graphics'/>.
    </para>""" )

_graphicsmenu.addItem(OOFMenuItem("New", callback=openGfx,
                                  help="Open a new graphics window.",
                                  accel='g',
                                  discussion="<para>Create a new <link linkend='Chapter:Graphics'>Graphics Window</link>.</para>"))

_windowmenu.addItem(_graphicsmenu)

_windowmenu.addItem(OOFMenuItem(
    'Layer_Editor', callback=dummy,
    help="Open or raise the Layer Editor window.",
    discussion="""<para>
Raise the <link linkend='Section:Graphics:LayerEditor'>Layer
Editor</link> window, if it has already been opened. If not, open it.
</para>"""
    ))

_windowmenu.addItem(OOFMenuItem(
    'Activity_Viewer', callback=dummy,
    help="Open or raise the Activity Viewer window.",
    discussion="""<para>
Raise the <link linkend='Section:Windows:ActivityViewer'>Activity
Viewer</link> window, if it is open. If not, open it.
</para>"""
    ))

#################################

## Help menu

helpmenu = OOF.addItem(OOFMenuItem('Help', help_menu=1))

debugmenu = helpmenu.addItem(OOFMenuItem(
    'Debug',
    discussion=
    """
<para>Tools for figuring out what's going on when it's not going well.
Mostly of interest to the developers.</para>
"""
    ))

def set_debug(menuitem, state):
    if state:
        debug.set_debug_mode()
    else:
        debug.clear_debug_mode()
        
debugmenu.addItem(CheckOOFMenuItem(
    'Debug',
    debug.debug(),
    callback=set_debug,
    help='Turn debugging mode on and off.',
    discussion=xmlmenudump.loadFile('DISCUSSIONS/common/IO/menu-debug.xml')
    ))

debugmenu.addItem(CheckOOFMenuItem(
    'Verbose_Switchboard',
    switchboard.switchboard.verbose,
    callback=switchboard.verbose,
    help='Print all switchboard calls as they occur.',
    discussion=xmlmenudump.loadFile('DISCUSSIONS/common/IO/menu-verbosesb.xml')
    ))

def setWarnPopups(menuitem, value):
    reporter.messagemanager.set_warning_pop_up(value)

helpmenu.addItem(CheckOOFMenuItem('Popup_warnings',1,
                                  ordering=-8,
                                  callback=setWarnPopups,
                                  help="Display warnings in a pop-up window or just in the message window?"))

####################################

## Profiling functions, in the Debug menu

profmenu = debugmenu.addItem(OOFMenuItem('Profile'))
prof = None

def profile_start(menuitem, filename, fudge):
    from common.EXTRA import profiler
    global prof
    prof = profiler.Profiler(filename, fudge=fudge)

def profile_stop(menuitem):
    prof.stop()
    
profmenu.addItem(OOFMenuItem(
    'Start',
    callback=profile_start,
    threadable = oofmenu.UNTHREADABLE,
    params=[StringParameter('filename', 'prof.out', tip="File name."),
            FloatParameter('fudge', 9e-6, tip="Fudge factor.")],
    help="Begin measuring execution time.",
    discussion=xmlmenudump.loadFile('DISCUSSIONS/common/IO/menu-profileStart.xml')
    ))
    
profmenu.addItem(OOFMenuItem(
    'Stop',
    callback=profile_stop,
    threadable = oofmenu.UNTHREADABLE,
    help="Stop measuring execution time.",
    discussion="<para>Stop profiling, and save the data in the file specified in <xref linkend='MenuItem:OOF.Help.Debug.Profile.Start'/>.</para>"
                             ))

def proffudge(menuitem, iterations):
    from common.EXTRA import profiler
    fudge = profiler.calibrate_profiler(iterations)
    helpmenu.Debug.Profile.Start.get_arg('fudge').value = fudge
    reporter.report('fudge =', fudge)

profmenu.addItem(OOFMenuItem(
    'FudgeFinder',
    callback=proffudge,
    threadable = oofmenu.UNTHREADABLE,
    params=[IntParameter('iterations', 1000, tip="Number of iterations.")],
    help='Find the factor to compensate for time spent in the profiler itself.',
    discussion="""
    <para>Find the machine dependent fudge factor that the profiler
    uses to compensate for function calling overhead in the profiler
    itself, by measuring how long it takes to call the profiler
    <varname>iterations</varname> times.  </para>
    """
    ))




####################################

## The following functions are included only if --debug is provided on
## the command line at start up.  Users shouldn't be interested in
## them.

if debug.debug():
    errmenu = helpmenu.addItem(OOFMenuItem('Error'))

    from SWIG.common import cdebug
    def _segfault(menuitem, delay):
        cdebug.segfault(delay)

    errmenu.addItem(OOFMenuItem('SegFault', callback=_segfault,
                                 threadable=oofmenu.THREADABLE,
                                 params=[IntParameter('delay', 10,
                                                      tip="Delay time.")],
                                 help='For external use only.  Slippery when wet.'))

    def _pyerror(menuitem):
        debug.fmsg()
        if reporter.query("Really?", "Yes", "No!") == "Yes":
            # os.abort()
            # raise ooferror.ErrPyProgrammingError("Blam!")
            reporter.warn("Look out!  Behind you!")
            raise AttributeError("Oops!")

    errmenu.addItem(OOFMenuItem('PyError', callback=_pyerror,
                                 threadable=oofmenu.THREADABLE,
                                 help='Do not taunt PyError.'))

    def _cerror(menuitem):
        debug.fmsg()
        if reporter.query("Really?", "Yes", "No!") == "Yes":
            cdebug.throwException()

    errmenu.addItem(OOFMenuItem('CError', callback=_cerror,
                                 threadable=oofmenu.THREADABLE))

    def _cpyerror(menuitem):
        debug.fmsg()
        if reporter.query("Really?", "Yes", "No!") == "Yes":
            cdebug.throwPythonException()

    errmenu.addItem(OOFMenuItem('CPyError', callback=_cpyerror,
                                threadable=oofmenu.THREADABLE))

    def loop(menuitem):
        while 1:
            pass
        debug.fmsg("What am I doing here?")

    errmenu.addItem(OOFMenuItem('Infinite_Loop', callback=loop,
                                 threadable=oofmenu.THREADABLE,
                                 help="I hope you have lots of time."))

    import os
    from SWIG.common import lock
    import time

    rw = lock.RWLock()

    def _py_read(menuitem, seconds):
        global rw
        rw.read_acquire()
        print "Got read permission for %d seconds." % seconds
        time.sleep(seconds)
        print "Releasing read."
        rw.read_release()

    def _py_write(menuitem, seconds):
        global rw
        rw.write_acquire()
        print "Got write permission for %d seconds." % seconds
        time.sleep(seconds)
        print "Releasing write."
        rw.write_release()

    debugmenu.addItem(OOFMenuItem('RWLock_read', callback=_py_read,
                                  threadable=oofmenu.THREADABLE,
                                  params=[IntParameter('seconds', 10,
                                                       tip="Sleeping time.")],
                                  help='Safe when used as directed.  For entertainment purposes only.'))

    debugmenu.addItem(OOFMenuItem('RWLock_write', callback=_py_write,
                                  threadable=oofmenu.THREADABLE,
                                  params=[IntParameter('seconds', 10,
                                                       tip="Sleeping time.")],
                                  help='Packaged by weight, contents may settle during shipping.'))


    def _wait(menuitem, seconds):
        cdebug.wait(seconds)

    debugmenu.addItem(OOFMenuItem('Wait', callback=_wait,
                                  threadable=oofmenu.THREADABLE,
                                  params=[IntParameter('seconds', 10,
                                                       tip="Waiting time.")],
                                  help='For internal use only.'))

