# -*- python -*-
# $RCSfile: demomesh.py,v $
# $Revision: 1.27 $
# $Author: reida $
# $Date: 2003/04/02 14:52:08 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

# Function that creates a simple mesh.  This is just for debugging and
# code development purposes.  For internal use only.  Not for
# distribution.  Void where prohibited.  All rights reversed.  Your
# milage may vary.

#from common import primitives
def demomesh(meshsize=10):
    eltypes = getMasterElementList()
    thermmaterial = newMaterial(
        "thermal",
        IsoHeatConductivity(name="heatcond", kappa=1.0),
        IsoElasticity(name="isoelastic", cijkl=CijIsoCijkl(c11=1.0, c12=0.1)),
        ColorProperty(name="paisley", color=RGBColor(red=0.6, green=0.7, blue=0.8)),
#                                ThermalExpansion('expand', 1., 1., 1., 0.),
        Orientation(name="unrotated", o=Abg(alpha=0.,beta=0.,gamma=0.))
        )
    gm = initial_skeleton('quad',meshsize, meshsize, 1.0, 1.0, None,
                            skeleton.moderate,thermmaterial)
    for e in gm.elements:
        e.material=gm.material
#    mesh =  gm.mesh(triangle=eltypes['Subparametric 6-node triangle'])
    mesh =  gm.mesh(quad=eltypes['Subparametric 8-node quadrilateral'])
#    mesh =  gm.mesh(quad=eltypes['Superparametric 8-node quadrilateral'])
#    mesh =  gm.mesh(quad=eltypes['Isoparametric 8-node quadrilateral'])
#    mesh =  gm.mesh(quad=eltypes['Isoparametric 4-node quadrilateral'])

    mesh.define_field(temperature)
    mesh.define_field(displacement)
    mesh.activate_field(displacement)
    temperature.set_in_plane(mesh, 1)
    displacement.set_in_plane(mesh, 1)
    mesh.activate_field(temperature)
    mesh.activate_equation(heat_eqn)
    mesh.activate_equation(forcebalance_eqn)
    fixT = mesh.newFixedBC(temperature, 0, heat_eqn, 0)
    fixX = mesh.newFixedBC(displacement, 0, forcebalance_eqn, 0)
    fixY = mesh.newFixedBC(displacement, 1, forcebalance_eqn, 1)
    mesh.getBoundary('bottom').addCondition(fixT, ContinuumProfile('0.0'))
    mesh.getBoundary('top').addCondition(fixT, ContinuumProfile('x'))
    mesh.getBoundary('left').addCondition(fixX, ContinuumProfile('0.0'))
    mesh.getBoundary('left').addCondition(fixY, ContinuumProfile('0.0'))
    mesh.getBoundary('right').addCondition(fixX, ContinuumProfile('0.1*y'))
    mesh.getBoundary('right').addCondition(fixY, ContinuumProfile('0.0'))
    return mesh

def demomeshMS(meshsize=10):
    MS=Microstructure("blackANDwhite",(100,100),(1.0,1.0))
    MS.getGroup("white")
    MS.getGroup("black")
    for j in range(100):
        for i in range(100):
            if (j-49)**2+(i-49)**2<=25**2:
                MS.addToGroup("black",[iPoint(j,i)])
            else: MS.addToGroup("white",[iPoint(j,i)])
    eltypes = getMasterElementList()
    thermmaterial = newMaterial(
        "thermal",
        IsoHeatConductivity(name="heatcond", kappa=1.0),
        IsoElasticity(name="isoelastic", cijkl=CijIsoCijkl(c11=1.0, c12=0.1)),
        ColorProperty(name="paisley", color=RGBColor(red=0.6, green=0.7, blue=0.8)),
#                                ThermalExpansion('expand', 1., 1., 1., 0.),
        Orientation(name="unrotated", o=Abg(alpha=0.,beta=0.,gamma=0.))
        )
    gm = initial_skeleton('quad',meshsize, meshsize, MS,
                          skeleton.moderate,thermmaterial)
    for e in gm.elements:
        e.material=gm.material
#    mesh =  gm.mesh(triangle=eltypes['Subparametric 6-node triangle'])
    mesh =  gm.mesh(quad=eltypes['Subparametric 8-node quadrilateral'])
#    mesh =  gm.mesh(quad=eltypes['Superparametric 8-node quadrilateral'])
#    mesh =  gm.mesh(quad=eltypes['Isoparametric 8-node quadrilateral'])
#    mesh =  gm.mesh(quad=eltypes['Isoparametric 4-node quadrilateral'])

    mesh.define_field(temperature)
    mesh.define_field(displacement)
    mesh.activate_field(displacement)
    temperature.set_in_plane(mesh, 1)
    displacement.set_in_plane(mesh, 1)
    mesh.activate_field(temperature)
    mesh.activate_equation(heat_eqn)
    mesh.activate_equation(forcebalance_eqn)
    fixT = mesh.newFixedBC(temperature, 0, heat_eqn, 0)
    fixX = mesh.newFixedBC(displacement, 0, forcebalance_eqn, 0)
    fixY = mesh.newFixedBC(displacement, 1, forcebalance_eqn, 1)
    mesh.getBoundary('bottom').addCondition(fixT, ContinuumProfile('0.0'))
    mesh.getBoundary('top').addCondition(fixT, ContinuumProfile('x'))
    mesh.getBoundary('left').addCondition(fixX, ContinuumProfile('0.0'))
    mesh.getBoundary('left').addCondition(fixY, ContinuumProfile('0.0'))
    mesh.getBoundary('right').addCondition(fixX, ContinuumProfile('0.1*y'))
    mesh.getBoundary('right').addCondition(fixY, ContinuumProfile('0.0'))
    return mesh
