# -*- python -*-
# $RCSfile: masterelement_.py,v $
# $Revision: 1.16 $
# $Author: haan $
# $Date: 2004/11/12 23:03:22 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from common import enum
from common import debug

# Global function that gets the master element array from C and wraps
# it in the right Python shadow object.  While doing so, it does some
# precomputation in MasterElementPtr.__init__().

_masterElementDict = {}

def getMasterElementList():
    lst = masterElementList()
    for me in lst:
        me = MasterElementPtr(me)
        if not _masterElementDict.has_key(me.name()):
            _masterElementDict[me.name()] = me
    return _masterElementDict

# Export this function to restricted namespace.
from common import utils
utils.OOFdefine('getMasterElementList', getMasterElementList)

########################

_masterElementEnumDict = {}
_enumunsorted = {}

def getMasterElementEnums():
    # Create and update Enum classes for MasterElement types.  There
    # is a separate enum class for each element geometry (number of
    # sides).  This function returns a dictionary containing the
    # classes, keyed by the number of sides of the elements.
    edict = getMasterElementList()
    for name, el in edict.items():
        try:
            enumclass = _masterElementEnumDict[el.nsides()]
        except KeyError:
            utils.OOFexec('from common import enum')
            utils.OOFexec('class Element%d(enum.EnumClass()):pass'%el.nsides())
            enumclass = utils.OOFeval('Element%d' % el.nsides())
            utils.OOFexec('Element%d.tip="%d sided FE elements."' % (el.nsides(), el.nsides()))
            _masterElementEnumDict[el.nsides()] = enumclass
            enumclass._sorted = 1
        if name not in enumclass.names:
            enum.addEnumName(enumclass, name, el.description())
            enumclass._sorted = 0
    for enumclass in _masterElementEnumDict.values():
        if not enumclass._sorted:
            enumclass.names.sort(melcompare)
            enumclass._sorted = 1
    return _masterElementEnumDict

def getMasterElementFromEnum(elementenum):
    edict = getMasterElementList()
    return edict[elementenum.name]

def melcompare(a, b):
    # MasterElement comparison function for determining the order in
    # which the elements are listed in the UI.
    edict = getMasterElementList()
    ela = edict[a]
    elb = edict[b]
    if ela.nnodes() < elb.nnodes(): return -1
    if ela.nnodes() > elb.nnodes(): return 1
    if ela.nmapnodes() < elb.nmapnodes(): return -1
    if ela.nmapnodes() > elb.nmapnodes(): return 1
    if ela.nfuncnodes() > elb.nfuncnodes(): return -1
    if ela.nfuncnodes() < elb.nfuncnodes(): return 1
    return 0

########################

# Make lists of nodes that have to be created -- these are in a
# dictionary indexed by the edge number, plus a special entry indexed
# by the string 'interior'.  Corner nodes are entirely omitted from
# the indexing scheme because they're created automatically.

def _MasterElementPtr___init__(self,this):
    self.this = this
    self.thisown = 0
    self.protodic = {'interior':[]}
    for i in range(self.nsides()):
        self.protodic[i] = []
    for i in range(self.nnodes()):
        proto = self.get_protonode(i)
        n = proto.nedges()              # number of edges this node is on
        if n == 0:
            self.protodic['interior'].append(proto)
        elif n == 1:
            edgeno = proto.getedge(0)
            self.protodic[edgeno].append(proto)
                
MasterElementPtr.__init__ = _MasterElementPtr___init__


### Is a given point on the boundary of the MasterElement?

##def _MasterElementPtr_isbdy(self, *pts):
##    # Pass in x and y separately, since this may be used with objects
##    # that aren't MasterCoords
##    if len(pts)==1:
##        pt = pts[0]
##        if isinstance(pt, MasterCoordPtr):
##            return self.onBoundary(pt)
##        return self.onBoundary(MasterCoord(pt.x, pt.y))
##    elif len(pts) == 2:
##        pt0 = pts[0]
##        pt1 = pts[1]
##        if isinstance(pt0, MasterCoordPtr) and isinstance(pt1, MasterCoordPtr):
##            return self.onBoundary2(pt0, pt1)
##        else:
##            return self.onBoundary2(MasterCoord(pt0.x, pt0.y),
##                                    MasterCoord(pt1.x, pt1.y))
##    else:
##        raise TypeError("Wrong number of arguments to MasterElement.isbdy()")

##MasterElementPtr.isbdy = _MasterElementPtr_isbdy

from SWIG.engine.elements import ElementPtr
from SWIG.engine.mastercoord import MasterCoordPtr
