// -*- C++ -*-
// $RCSfile: mvmult.C,v $
// $Revision: 1.5 $
// $Author: reida $
// $Date: 2003/04/02 14:52:26 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>

#include <assert.h>
#include "mvmult.h"

// blas declarations
extern "C" {
  void dgemm_(char*, char*, int*, int*, int*, double*,
	      const double*, int*, const double*, int*, double*, double*, int*);
  void dgemv_(char*, int*, int*, double*, const double*, int*, const double*,
	      int*, double*, double*, int*);
  void daxpy_(int*, double*, const double*, int*, double*, int*);
}

// A * x

VECTOR_D operator*(const MATRIX_D &m, const VECTOR_D &v) {
  int vsize = v.size();
  int mrows = m.nrows();
  int mcols = m.ncols();
  assert(vsize == mcols);

  VECTOR_D result(mrows, 0.0);
  // need to pass addresses of constants to fortran BLAS routines
  char enn = 'n';
  double One = 1.0;
  int one = 1;
  double zero = 0.0;

  dgemv_(&enn, &mrows, &mcols, &One, &m(0, 0), &mrows, &v[0], &one, &zero, 
	 &result[0], &one);
  return result;
}

// A^T * x

VECTOR_D trans_mult(const MATRIX_D &m, const VECTOR_D &v) {
  int vsize = v.size();
  int mrows = m.nrows();
  int mcols = m.ncols();
  assert(vsize == mrows);

  VECTOR_D result(mcols, 0.0);
  // need to pass addresses of constants to fortran BLAS routines
  char tee = 't';
  double One = 1.0;
  int one = 1;
  double zero = 0.0;

  dgemv_(&tee, &mrows, &mcols, &One, &m(0, 0), &mrows, &v[0], &one, &zero, 
	 &result[0], &one);
  return result;
}

// A * B

MATRIX_D operator*(const MATRIX_D &a, const MATRIX_D &b) {
  int arows = a.size(0);
  int acols = a.size(1);
  int brows = b.size(0);
  int bcols = b.size(1);
  MATRIX_D product(arows, bcols);
  // call blas	
  char enn = 'n';
  double one = 1.0;
  double zero = 0.0;
  dgemm_(&enn, &enn, &arows, &bcols, &acols, &one, &a(0, 0), &arows,
	 &b(0, 0), &brows, &zero, &product(0, 0), &arows);
  return product;
}

// A^T * B

MATRIX_D trans_mult(const MATRIX_D &a, const MATRIX_D &b) {
  int arows = a.size(0);
  int acols = a.size(1);
  int brows = b.size(0);
  int bcols = b.size(1);
  MATRIX_D product(acols, bcols);
  char enn = 'n';
  char tee = 't';
  double one = 1.0;
  double zero = 0.0;
  dgemm_(&tee, &enn, &acols, &bcols, &arows, &one, &a(0, 0), &arows, 
	 &b(0, 0), &brows, &zero, &product(0, 0), &acols);
  return product;
}

// A * B^T

MATRIX_D mult_trans(const MATRIX_D &a, const MATRIX_D &b) {
  int arows = a.size(0);
  int acols = a.size(1);
  int brows = b.size(0);
  int bcols = b.size(1);
  MATRIX_D product(acols, bcols);
  char enn = 'n';
  char tee = 't';
  double one = 1.0;
  double zero = 0.0;
  dgemm_(&enn, &tee, &acols, &bcols, &arows, &one, &a(0, 0), &arows, 
	 &b(0, 0), &brows, &zero, &product(0, 0), &acols);
  return product;
}

// A * M * A^T

MATRIX_D transform(const MATRIX_D &m, const MATRIX_D &a) {
  // probably not the most efficient way to do it...
  return a * mult_trans(m, a);
}
