# -*- python -*-
# $RCSfile: outputval_.py,v $
# $Revision: 1.4 $
# $Author: reida $
# $Date: 2005/01/31 21:14:09 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 



_labels = ["X-component", "Y-component", "Z-component"]

# Header and representation data for OutputValPtr classes ---
# used by analysis machinery to format the output.

# Return appropriate subset of "X", "Y", "Z" if size is less than
# three, otherwise return numeric component indices starting from 1.
def _VectorOutputValPtr_label_list(self):
    s = self.size()
    if s <= len(_labels):
        return _labels[:s]
    else:
        return [`x`+'-component' for x in range(1,s+1)]

VectorOutputValPtr.label_list = _VectorOutputValPtr_label_list

def _ScalarOutputValPtr_label_list(self):
    return ["Value"]

ScalarOutputValPtr.label_list = _ScalarOutputValPtr_label_list


#################################################################




# Arithemtic operations for the OutputValPtr subclasses.  These must
# create a new object which Python has ownership of, otherwise data
# can get lost.  The preliminary "add" step could be replaced by a
# smarter VectorOutputVal constructor.

def _VectorOutputValPtr___add__(self, other):
    res = VectorOutputVal(self.size())
    res._add(self)
    res._add(other)
    return res

VectorOutputValPtr.__add__ = _VectorOutputValPtr___add__


def _VectorOutputValPtr___sub__(self, other):
    res = VectorOutputVal(self.size())
    res._add(self)
    res._sub(other)
    return res

VectorOutputValPtr.__sub__ = _VectorOutputValPtr___sub__


def _ScalarOutputValPtr___add__(self, other):
    res = ScalarOutputVal(0.0)
    res._add(self)
    res._add(other)
    return res

ScalarOutputValPtr.__add__ = _ScalarOutputValPtr___add__


def _ScalarOutputValPtr___sub__(self, other):
    res = ScalarOutputVal(0.0)
    res._add(self)
    res._sub(other)
    return res

ScalarOutputValPtr.__sub__ = _ScalarOutputValPtr___sub__




# Multiplication and division only defined for scalar operands, of course.

def _VectorOutputValPtr___mul__(self, other):
    res = VectorOutputVal(self.size())
    res._add(self)
    res._mul(other)
    return res

VectorOutputValPtr.__mul__ = _VectorOutputValPtr___mul__


def _VectorOutputValPtr___div__(self, other):
    res = VectorOutputVal(self.size())
    res._add(self)
    res._mul(1.0/other)
    return res

VectorOutputValPtr.__div__ = _VectorOutputValPtr___div__


def _ScalarOutputValPtr___mul__(self, other):
    res = ScalarOutputVal(0.0)
    res._add(self)
    res._mul(other)
    return res

ScalarOutputValPtr.__mul__ = _ScalarOutputValPtr___mul__


def _ScalarOutputValPtr___div__(self, other):
    res = ScalarOutputVal(0.0)
    res._add(self)
    res._mul(1.0/other)
    return res

ScalarOutputValPtr.__div__ = _ScalarOutputValPtr___div__
