# -*- python -*-
# $RCSfile: profilefunction.py,v $
# $Revision: 1.10 $
# $Author: langer $
# $Date: 2004/12/08 18:13:40 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov.

from common import strfunction
from common import utils
from common.IO import parameter
from common.IO import xmlmenudump
import struct
import types


# String-based function object used by boundary profiles -- user types
# in a string, and one of these objects gets built.  Profiles'
# __call__ methods provide the relevant arguments.

class ProfileFunction(strfunction.StrFunction):
    def __init__(self, funcstr):
        # x and y are the current location and is guaranteed to be
        # provided. All others are provided if applicable.  i is the
        # node index, s is the arc length, alpha is the fractional arc
        # length.
        ## TODO LATER: Add the boundary normal to the parameters.
        strfunction.StrFunction.__init__(
            self, 'x, y, i, s, alpha', funcstr)
    def __repr__(self):
        # ProfileFunctions only have to write their values when they
        # are used in ProfileFunctionParameters, which can set
        # themselves from a string. So just write the string.
        return "'%s'" % self.funcstr
##        return "ProfileFunction('%s')" % self.funcstr

xmlmenudump.XMLObjectDoc(
    'ProfileFunction',
    xmlmenudump.loadFile('DISCUSSIONS/engine/profilefunction.xml'))

# The ProfileFunctionParameter can accept either a string or a
# ProfileFunction object, but it only stores a ProfileFunction object.
# Like XYStrFunctionParameter, it needs a special "set" function.

class ProfileFunctionParameter(parameter.ParameterBase):
    def __init__(self, name, value=None, default=None, tip=None):
##        self.checker = parameter.TypeChecker(ProfileFunction)
        parameter.ParameterBase.__init__(self, name,
                                         value=value,
                                         default=default, tip=tip)
    def checker(self, x):
        if type(x) is types.StringType:
            return
        if isinstance(x, ProfileFunction):
            return
        parameter.raiseTypeError(type(x), 'string or ProfileFunction')
    def set(self, value):
        if type(value) is types.StringType:
            self.__dict__['value'] = ProfileFunction(value)
        elif isinstance(value, ProfileFunction):
            self.__dict__['value'] = value
        elif value is None:
            self.__dict__['value'] = None
        else:
            raise parameter.ParameterMismatch(
                'Got ' + `value` + ' for Parameter ' + self.name)
    def binaryRepr(self, datafile, value):
        length = len(value.funcstr)
        return struct.pack('>i', length) + value.funcstr
    def binaryRead(self, parser):
        b = parser.getBytes(struct.calcsize('>i'))
        (length,) = struct.unpack('>i', b)
        return ProfileFunction(parser.getBytes(length))
    def valueDesc(self):
        return "A <link linkend='Object:ProfileFunction'><classname>ProfileFunction</classname></link> object."

utils.OOFdefine('ProfileFunction', ProfileFunction)

# TODO: Make it possible for a ProfileFunctionParameter to accept a
# string, not just a ProfileFunction, like the way
# XYStrFunctionParameter can accept a string.  Then make the repr for
# ProfileFunction be just the string.  This will simplify the scripts:
# ContinuumProfile(ProfileFunction('x*y.)) will become
# ContinuumProfile('x*y').

#  Fix the documentation when this change is made.  There's an example
#  presented in reg-dirichlet.xml.
