// -*- C++ -*-
// $RCSfile: thermo.h,v $
// $Revision: 1.10 $
// $Author: langer $
// $Date: 2004/03/16 16:45:37 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#ifndef THERMOELASTICITY_H
#define THERMOELASTICITY_H

#include "engine/property/elasticity/elasticity.h"
#include "engine/property/elasticity/cijkl.h"
#include <string>

class ScalarField;

class ThermoElasticityProp : public Elasticity {
private:
  Cijkl cijkl_;
  double tzero_;   // Temperature at which mu is unchanged.
  double dmudt_;   // Change in mu per unit change in T.
public:
  ThermoElasticityProp(PyObject *registry, const std::string &name, 
		       Cijkl *c, double tzero, double dmudt);
  virtual ~ThermoElasticityProp() {}
  Cijkl *cijkl_pointer() { return &cijkl_; }
  const Cijkl cijkl(const FEMesh *, const Element *, 
		    const MasterPosition &) const;
  double t0() const { return tzero_; }
  double dmudt() const { return dmudt_; }

protected:
  ScalarField *temperature;
};

#endif
