// -*- C++ -*-
// $RCSfile: skeletonrelaxationrate.h,v $
// $Revision: 1.1 $
// $Author: redwing $
// $Date: 2005/02/24 21:25:33 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#ifndef SKELETONRELAXATIONRATE_H
#define SKELETONRELAXATIONRATE_H


#include "engine/matvec.h"
#include "engine/property.h"
#include "engine/symmmatrix.h"
#include <string>

class Elasticity;
class Element;
class Flux;
class FluxData;
class Material;
class Position;
class OrientationProp;
class OutputVal;
class PropertyOutput;
class ScalarField;
class SymmetricTensorFlux;

// Property analogous to plasticity.
// Its purpose is to expand/contract the skeleton
// element node positions, based on the quality of the mesh
// energy shape/homogeneity.

class SkeletonRelaxationRate : public Property {
private:
  // Physical parameters.
  double alpha_; 
  double beta_;
  Elasticity *elasticity;	// pointer to this Material's Elasticity
  SymmetricTensorFlux *stress_flux;
  SymmMatrix3 shapetensor(const Element*, const FEMesh *) const;
public:
  SkeletonRelaxationRate(PyObject *registry, const std::string &nm, 
		       double alpha, double beta);
  virtual ~SkeletonRelaxationRate() {}
  virtual void cross_reference(Material*);
  virtual void precompute();

  virtual void fluxmatrix(const FEMesh*, const Element*, 
			  const ElementFuncNodeIterator&,
			  const Flux*, FluxData*, const MasterPosition&) const;
  virtual void fluxrhs(const FEMesh*, const Element*,
		       const Flux*, FluxData*, const MasterPosition&) const;
  
  virtual int K_integration_order(int) const;

  const double alpha() const { return alpha_; }
  virtual int is_symmetric(const FEMesh*) const;
};

#endif
