// -*- C++ -*-
// $RCSfile: pypropertywrapper.h,v $
// $Revision: 1.21 $
// $Author: redwing $
// $Date: 2004/12/17 18:03:48 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>

// Structure to wrap a pure-Python property, and make it available by
// way of the usual interface to the stiffness-matrix construction
// process.

#include <Python.h>
#include "engine/element.h"
#include "property.h"
#include "common/pythonexportable.h"

class Field;
class Flux;
class Material;
class FEMesh;
class ElementFuncNodeIterator;
class MasterPosition;
using namespace std;


// Data storage class for objects descended from PyPropertyWrapper.
class PyPropertyElementData : public ElementData {
private:
  PyObject *_data;
  static std::string classname_;
  static std::string modulename_;
public:
  PyPropertyElementData(const std::string &name, PyObject *dat);
  virtual ~PyPropertyElementData();
  virtual const std::string &classname() const { return classname_; }
  virtual const std::string &modulename() const { return modulename_; }
  PyObject *data();
};

class PyPropertyWrapper : public Property, 
			  virtual public PythonNative<Property> {
public:
  PyPropertyWrapper(PyObject *referent, PyObject *regstn, const string &name);
  virtual ~PyPropertyWrapper();
  
  // The following functions involve callbacks, and are defined
  // in the .swg file.
  virtual void precompute();
  virtual void cross_reference(Material*);
  virtual void begin_element(const Element*);
  virtual void end_element(const Element*);
  virtual void post_process(FEMesh *, const Element*) const;
  virtual int K_integration_order(int) const;
  virtual void fluxmatrix(const FEMesh*, const Element*,
			  const ElementFuncNodeIterator&,
			  const Flux*, FluxData*,
			  const MasterPosition&) const;
  virtual void fluxrhs(const FEMesh*, const Element*, const Flux*, FluxData*,
		       const MasterPosition&) const;

//   // The PyPropertyWrapper also needs to provide an interface to the 
//   // element's data storage and retrieval.
//   int appendElementData(Element *el, PyPropertyElementData *data);
//   PyPropertyElementData *getElementData(Element *el, int i);
//   int getElementDataIndex(Element *el, std::string name);
  virtual int is_symmetric(const FEMesh* mesh) const {return 1;}
private:
  PyObject *referent_;      // Pointer to the actual Python object.
};

